package de.trantor.kroll.midp.crypto;

import javax.microedition.midlet.MIDlet;
import javax.microedition.lcdui.*;

import org.bouncycastle.util.encoders.*;

import org.bouncycastle.crypto.*;
import org.bouncycastle.crypto.engines.*;
import org.bouncycastle.crypto.paddings.*;
import org.bouncycastle.crypto.modes.*;
import org.bouncycastle.crypto.params.*;

public class BlowfishTest extends MIDlet implements CommandListener {

	private long t1 = 0;
	private long t2 = 0;

	private Display display = null;
	private boolean encrypted = false;

	private String key = "abcdefghijklmnopqrstuvwxyz0123456789abcdefghijklmnopqrstuvwxyz0123456789";
	private String text = "This is the J2ME Crypto Test String";

	private BufferedBlockCipher bfCipher = null;

	private Form mainForm = null;

	private StringItem statusItem = new StringItem("Status:", "init");
	private StringItem keyItem = new StringItem("Key:", key);
	private StringItem textItem = new StringItem("Text:", text);

	public static Command encrypt = new Command("Encrypt", Command.SCREEN, 1);
	public static Command decrypt = new Command("Decrypt", Command.SCREEN, 2);

	public BlowfishTest() throws CryptoException {

		mainForm = new Form("Blowfish Test");

		mainForm.append(statusItem);
		mainForm.append(textItem);
		mainForm.append(keyItem);

		mainForm.addCommand(encrypt);
		mainForm.addCommand(decrypt);
		mainForm.setCommandListener(this);
	}

	public void startApp() {
		display = Display.getDisplay(this);
		display.setCurrent(mainForm);
	}

	public void pauseApp() {
	}

	public void destroyApp(boolean unconditional) {
	}

	public void commandAction(Command c, Displayable d) {
		if (c == encrypt && !encrypted) {
			try {
				t1 = System.currentTimeMillis();
				String enc = encryptText(key, text);
				t2 = System.currentTimeMillis();
				textItem.setText(enc);
				statusItem.setText("encrypted t="  + (t2-t1) + "ms");
				encrypted = true;
			}
			catch (CryptoException ce) {
				throw new RuntimeException(ce.toString());
			}
		}
		else if (c == decrypt && encrypted) {
			try {
				String dec = decryptText(key, textItem.getText());
				t1 = System.currentTimeMillis();
				textItem.setText(dec);
				t2 = System.currentTimeMillis();
				statusItem.setText("decrypted t="  + (t2-t1) + "ms");
				encrypted = false;
			}
			catch (CryptoException ce) {
				throw new RuntimeException(ce.toString());
			}
		}
	}

	private String encryptText(String key, String text) throws CryptoException {
		byte[] keyBytes = Hex.decode(key.getBytes());
		byte[] ptBytes = text.getBytes();

		bfCipher = new PaddedBufferedBlockCipher(new CBCBlockCipher(new BlowfishEngine()));

		bfCipher.init(true, new KeyParameter(keyBytes));
		byte[] result = new byte[bfCipher.getOutputSize(ptBytes.length)];
		int len = bfCipher.processBytes(ptBytes, 0, ptBytes.length, result, 0);
		bfCipher.doFinal(result, len);
		return new String(Hex.encode(result));
	}

	private String decryptText(String key, String cipherText) throws CryptoException {
		byte[] keyBytes = Hex.decode(key.getBytes());
		byte[] textBytes = Hex.decode(cipherText.getBytes());

		bfCipher.init(false, new KeyParameter(keyBytes));

		byte[] result = new byte[bfCipher.getOutputSize(textBytes.length)];
		int len = bfCipher.processBytes(textBytes, 0, textBytes.length, result, 0);
		bfCipher.doFinal(result, len);
		return new String(result).trim();
	}
}

